# -*- coding: utf-8 -*-

"""
cli module
It parses args and runs what's needed if every over modules
depending of what command is done
"""

import configparser
import logging
import sys
from argparse import ArgumentParser, RawTextHelpFormatter
from datetime import datetime
from pathlib import Path
from typing import NoReturn

from rich.traceback import install

from tenkan.config import load_config
from tenkan.feedsfile import (
    add_feed,
    create,
    del_feed,
    list_feeds,
    update_last_run,
)
from tenkan.files import delete_folder
from tenkan.processing import (
    fetch_feeds,
    prepare_fetched_content,
    process_fetched_feeds,
    write_processed_feeds,
)

# rich tracebacks
install(show_locals=True)


class MyParser(ArgumentParser):  # pylint: disable=too-few-public-methods
    """Child class to print help msg if no or bad args given"""

    def error(self, message: str) -> NoReturn:
        """exit"""
        sys.stderr.write(f'error: {message}')
        self.print_help()
        sys.exit(2)


def load_args(args: list):
    """args parsing function"""

    desc = 'tenkan : RSS/atom feed converter from html to gemini\n\nTo show the detailed help of a COMMAND run `ytcc COMMAND --help`.'
    parser = MyParser(
        description=desc, prog='tenkan', formatter_class=RawTextHelpFormatter
    )

    parser.add_argument(
        '-v',
        '--version',
        action='version',
        version='%(prog)s 0.1.0',
        help='show %(prog)s version number and exit',
    )

    parser.add_argument(
        '--config',
        default=f'{str(Path.home())}/.config/tenkan/tenkan.conf',
        help='config file, $HOME/.config/tenkan/tenkan.conf by default',
        dest='config',
    )
    parser.add_argument(
        '--feedsfile',
        default=f'{str(Path.home())}/.config/tenkan/feeds.json',
        help='feeds file containing feed list, $HOME/.config/tenkan/feeds.json by default',
        dest='feedsfile',
    )

    parser.add_argument(
        '--debug', action='store_true', help='debug mode', dest='debug'
    )

    subparsers = parser.add_subparsers(
        title='command', required=True, dest='command'
    )

    parser_add = subparsers.add_parser(
        'add', help='add a feed to the feeds list'
    )
    parser_add.add_argument(
        'name', help='the name of the feed you want to add'
    )
    parser_add.add_argument('url', help='the HTTP url of the feed')

    parser_update = subparsers.add_parser(
        'update', help='update feeds folder from feed list'
    )
    parser_update.add_argument(
        '--force',
        action='store_true',
        default=False,
        help='update feed list even if there is no new content',
    )

    parser_list = subparsers.add_parser(
        'list', help='list all feeds in feeds list'
    )
    parser_list.add_argument(
        'list', help='list all feeds in feeds list', action='store_true'
    )

    parser_delete = subparsers.add_parser(
        'delete', help='remove a feed to the feeds list'
    )
    parser_delete.add_argument(
        'name', help='the name of the feed you want to delete'
    )
    parser_delete.add_argument(
        '--delete-gmi-folder',
        help='delete gmi folder, True by default',
        action='store_true',
        default=False,
        dest='delete_folder',
    )
    return parser.parse_args(args)


def set_logging(args, config: configparser.ConfigParser) -> None:
    """define logging settings"""
    log = logging.getLogger()
    log.setLevel(logging.INFO)
    if args.debug:
        log.setLevel(logging.DEBUG)

    console_formatter = logging.Formatter(fmt='%(message)s')
    file_formatter = logging.Formatter(
        fmt='%(asctime)s %(levelname)s: %(message)s'
    )

    stdout_handler = logging.StreamHandler(stream=sys.stdout)
    stdout_handler.setFormatter(console_formatter)
    log.addHandler(stdout_handler)

    if config['tenkan'].get('log_file'):
        file_handler = logging.FileHandler(
            filename=config['tenkan'].get('log_file'),
            encoding='utf-8',
        )
        file_handler.setFormatter(file_formatter)
        log.addHandler(file_handler)


def run(args, config: configparser.ConfigParser) -> None:
    """run stuff depending of command used"""
    # exit with error if json file not found with actions other than add
    if not Path(args.feedsfile).exists() and 'add' not in args.command:
        logging.error('No json file %s, can\'t continue', args.feedsfile)
        sys.exit(1)

    # list feeds in a pretty format
    if args.command == 'list':
        list_feeds(file=args.feedsfile)

    # add a feed to feeds file
    if args.command == 'add':
        # if home directory, creates json with empty structure if no file yet
        if not Path(args.feedsfile).parents[0].exists():
            if str(Path(args.feedsfile).parents[0]) == str(Path.home()):
                Path(args.feedsfile).parents[0].mkdir(
                    parents=True, exist_ok=True
                )
            else:
                logging.error(
                    'Directory of feeds file %s not found, exiting',
                )
                sys.exit(1)
        if not Path(args.feedsfile).is_file():
            create(args.feedsfile)
        add_feed(file=args.feedsfile, feed_name=args.name, feed_url=args.url)

    # delete a feed from feeds file
    if args.command == 'delete':
        del_feed(file=args.feedsfile, feed_name=args.name)
        if args.delete_folder:
            delete_folder(
                path=config['tenkan']['gemini_path'], feed_name=args.name
            )

    # update content
    if args.command == 'update':
        fetched_feeds = fetch_feeds(
            feeds_file=args.feedsfile,
            gmi_url=config['tenkan']['gemini_url'],
        )
        print('')
        fetched_feeds = prepare_fetched_content(fetched_feeds, args.force)
        feed_list = process_fetched_feeds(
            config=config,
            fetched_feeds=fetched_feeds,
            force=args.force,
        )
        if feed_list:
            write_processed_feeds(args, config, feed_list)
        else:
            logging.info('No new content to process, stopping')
        update_last_run(args.feedsfile, str(datetime.now()))


def main() -> None:
    """load conf, args, set logging and run main program"""

    args = load_args(args=sys.argv[1:])
    config = load_config(args.config)
    set_logging(args, config)
    run(args, config)



# -*- coding: utf-8 -*-

"""processing module : feeds file processing """

import configparser
import hashlib
import json
import logging
import os
from concurrent.futures import ThreadPoolExecutor

import feedparser  # type: ignore

from tenkan.feed import Feed
from tenkan.feedsfile import read, update_feed
from tenkan.files import path_exists, write_files
from tenkan.utils import display_feeds_fetch_progress, measure


@measure
def fetch_feeds(feeds_file: str, gmi_url: str) -> list:
    """Fetch all http feeds with threads"""
    workers = os.cpu_count() or 1
    try:
        fetched_feeds = []
        with ThreadPoolExecutor(max_workers=workers) as executor:
            for item, values in read(feeds_file)['feeds'].items():
                fetched_feeds.append(
                    {
                        'title': item,
                        'fetched_content': executor.submit(
                            feedparser.parse, values['url']
                        ),
                        'gmi_url': gmi_url,
                        'last_update': values['last_update'],
                        'fetched_hash_last_update': None,
                        'json_hash_last_update': values['hash_last_update'],
                    }
                )
            display_feeds_fetch_progress(fetched_feeds)
        return fetched_feeds
    except json.decoder.JSONDecodeError as bad_json:
        raise bad_json


@measure
def prepare_fetched_content(fetched_feeds: list, force: bool = False) -> list:
    """Prepare some necessary data to be sent to feed object"""
    list_to_export = []
    for ftfd in fetched_feeds:
        try:
            # store workers result into fetched_content
            ftfd['fetched_content'] = ftfd['fetched_content'].result()  # type: ignore
            # we store a sha256 footprint of fetched content,
            # to compare to last known footprint
            tmp_hash = hashlib.sha256(
                str(ftfd['fetched_content'].get('entries')[0]).encode()
            )
            if tmp_hash.hexdigest() != ftfd['json_hash_last_update'] or force:
                ftfd['fetched_hash_last_update'] = tmp_hash.hexdigest()
                list_to_export.append(ftfd)
        # sometimes we don't get anything in fetched_content, so just ignore it
        except IndexError:
            pass
    return list_to_export


@measure
def process_fetched_feeds(
    config: configparser.ConfigParser, fetched_feeds: list, force: bool = False
) -> list:
    """Process previously fetched feeds"""
    feed_list = []
    for ftfd in fetched_feeds:
        # initialize feed object
        feed = Feed(
            input_content=ftfd,
            filters=config['filters'],
            formatting=config['formatting'],
        )
        # process feeds if there are updates since last run
        # or if the feed had never been processed
        # or if --force option is used
        if (
            feed.needs_update()
            or not path_exists(
                path=config['tenkan']['gemini_path'] + ftfd['title']
            )
            or force
        ):
            logging.info('Processing %s', ftfd['title'])
            feed.get_new_entries()
            feed_list.append(feed.export_content())
    return feed_list


@measure
def write_processed_feeds(
    args, config: configparser.ConfigParser, feed_list: list
) -> None:
    """Write files from processed feeds into gemini folder"""
    for files_data in feed_list:
        write_files(
            path=config['tenkan']['gemini_path'],
            data=files_data,
            max_num_entries=int(
                config['tenkan'].get('purge_feed_folder_after', '9999')
            ),
        )
        update_feed(
            file=args.feedsfile,
            feed_name=files_data['title'],
            hash_last_update=files_data['hash_last_update'],
        )



# -*- coding: utf-8 -*-

""" files module : generated gemini feeds files management """

import logging
import pathlib
import shutil
from typing import Dict, Union

from feedgen.feed import FeedGenerator  # type: ignore


def path_exists(path: str) -> bool:
    """Check if feed path exists"""
    if pathlib.Path(path).is_dir():
        return True
    return False


def write_files(path: str, data: dict, max_num_entries: int) -> None:
    """
    Converts feed objects into files and write them in the feed folder
    """
    tpath = path
    path = path + data['title']
    pathlib.Path(path).mkdir(exist_ok=True)
    num_entries = 0
    # count entries in index file
    if pathlib.Path(f'{path}/index.gmi').is_file():
        num_entries = sum(1 for line in open(f'{path}/index.gmi'))

    # if there is more articles than defined in max_num_entries, delete and rewrite
    if num_entries > max_num_entries:
        delete_folder(tpath, data['title'])

    index_file_write_header(path, data['title'])
    urls = []
    art_output = {}
    for article in data['articles']:
        art_output = write_article(article, data, path)
        urls.append(art_output['url'])
    index_file_write_footer(path)
    # no need to update atom file if no new articles (write_article func returns url list)
    if art_output.get('new_file'):
        _rebuild_atom_file(path=path, data=data, urls=urls)


# def purge_folder(path: str) -> None:
#     """Purge folder with too many entries"""
#     logging.info('Purging %s folder', path)
#     files = [x for x in pathlib.Path(f'{path}').iterdir() if x.is_file()]
#     for file in files:
#         pathlib.Path.unlink(file)


def delete_folder(path: str, feed_name: str) -> None:
    """delete a feed folder"""
    if pathlib.Path(f'{path}{feed_name}/').exists():
        shutil.rmtree(f'{path}{feed_name}')
        logging.info('%s/%s folder deleted', path, feed_name)
    else:
        logging.info(
            'folder %s%s not present, nothing to delete', path, feed_name
        )


def index_file_write_header(path: str, title: str) -> None:
    """Write index header"""
    with open(f'{path}/index.gmi', 'w') as index:
        index.write(f'# {title}\n\n')
        index.write('=> ../ ..\n')


def index_file_write_footer(path: str) -> None:
    """Write index footer"""
    with open(f'{path}/index.gmi', 'a') as index:
        index.write('\n=> atom.xml Atom feed\n')


def write_article(
    article: dict, data: dict, path: str
) -> Dict[str, Union[bool, str]]:
    """Write individual article"""
    # prepare data for file format
    date = article['article_date']
    file_date = date.strftime('%Y-%m-%d_%H-%M-%S')
    date = date.strftime('%Y-%m-%d %H:%M:%S')
    file_title = article['article_formatted_title']
    content = article['article_content']

    # we add the entry into index file
    with open(f'{path}/index.gmi', 'a') as index:
        index.write(
            f"=> {file_date}_{file_title}.gmi {date} - {article['article_title']}\n"
        )

    new_file = False
    # write the file is it doesn't exist, obviously
    if not pathlib.Path(f'{path}/{file_date}_{file_title}.gmi').is_file():
        new_file = True
        logging.info('%s : adding entry %s', data['title'], file_title)
        # we write the entry file
        author = article['author'] if 'author' in article else None

        pathlib.Path(f'{path}/{file_date}_{file_title}.gmi').write_text(
            f"# {article['article_title']}\n\n=> {article['http_url']}\n\n{date}, {author}\n\n{content}"
        )
    url = f"{data['gmi_url']}{data['title']}/{file_date}_{file_title}.gmi"

    return {'new_file': new_file, 'url': url}


def _rebuild_atom_file(path: str, data: dict, urls: list) -> None:
    """rebuilds the atom file into gmi folder"""

    atomfeed = FeedGenerator()
    atomfeed.id(data['gmi_url'])
    atomfeed.title(data['title'])
    atomfeed.updated = data['last_update']
    atomfeed.link(href=f"{data['gmi_url']}.atom.xml", rel='self')
    atomfeed.link(href=data['gmi_url'], rel='alternate')

    # rebuild all articles
    for art, article in enumerate(data['articles']):
        atomentry = atomfeed.add_entry()
        url = urls[art]
        atomentry.guid(url)
        atomentry.link(href=url, rel='alternate')
        atomentry.updated(article['updated'])
        atomentry.title(article['article_title'])

    atomfeed.atom_file(f'{path}/atom.xml', pretty=True)
    logging.info('Wrote Atom feed for %s', data['title'])



# -*- coding: utf-8 -*-

"""utils module : various utils"""

import logging
from time import sleep, time


def display_feeds_fetch_progress(fetched_feeds: list) -> None:
    """Display feeds being fetched"""
    qsize = len(fetched_feeds)
    while True:
        done = len([x for x in fetched_feeds if x['fetched_content'].done()])
        print(f'Fetching feeds [{done}/{qsize}]', end='\r', flush=True)
        sleep(0.3)
        if done == qsize:
            break


def measure(func):
    """
    Decorator to measure time took by a func
    Used only in debug mode
    """

    def wrap_func(*args, **kwargs):
        time1 = time()
        result = func(*args, **kwargs)
        time2 = time()
        logging.debug(
            'Function %s executed in %ss', func.__name__, time2 - time1
        )
        return result

    return wrap_func



#!/usr/bin/python3
# -*- coding: utf-8 -*-

""" feedsfile mddule : json feeds file manipulation """

import json
import logging
from typing import Dict  # , Optional

from prettytable import PrettyTable


def create(file: str) -> None:
    """file creation"""
    with open(file, 'x') as _file:
        data: dict = {'feeds': {}}
        json.dump(data, _file)


def read(file: str) -> Dict[str, Dict[str, Dict[str, str]]]:
    """read file and return json data"""
    with open(file, 'r') as _file:
        file_data = json.load(_file)
    return file_data


def _write(file: str, file_data: Dict[str, Dict[str, Dict[str, str]]]) -> None:
    """write new data into file"""
    with open(file, 'w') as file_updated:
        json.dump(file_data, file_updated, indent=4)


def add_feed(file: str, feed_name: str, feed_url: str) -> None:
    """add a new feed into existing file"""
    file_data: Dict[str, Dict[str, Dict[str, str]]] = read(file)
    file_data['feeds'][feed_name] = {
        'url': feed_url,
        'last_update': '',
        'hash_last_update': '',
    }
    _write(file, file_data)
    logging.info('feed %s added', feed_name)


def del_feed(file: str, feed_name: str) -> None:
    """remove feed from file"""
    file_data = read(file)
    # don't do anything if no feed found
    if file_data['feeds'].get(feed_name):
        del file_data['feeds'][feed_name]
        _write(file, file_data)
        logging.info('feed %s deleted', feed_name)
    else:
        logging.info('no feed %s found into feeds file', feed_name)


def get_feed_item(file: str, feed_name: str, item: str) -> str:
    """Return element of a defined feed"""
    file_data = read(file)
    item = file_data['feeds'][feed_name][item]
    return item


def update_last_run(file: str, date: str) -> None:
    """Update last_run key in json file"""
    file_data: dict = read(file)
    file_data['last_run'] = date
    _write(file, file_data)


def update_feed(file: str, feed_name: str, hash_last_update: str) -> None:
    """update last update date of a defined feed"""
    file_data = read(file)
    file_data['feeds'][feed_name]['hash_last_update'] = hash_last_update
    _write(file, file_data)


def list_feeds(file: str) -> None:
    """list feed file content"""
    file_data = read(file)
    table = PrettyTable()
    table.field_names = ['Title', 'URL']
    for item, value in file_data['feeds'].items():
        table.add_row([item, value['url']])
    logging.info(table)



# -*- coding: utf-8 -*-

""" feed module : feed object """

import logging
import re
import sys
from datetime import datetime, timezone
from typing import List

import requests  # type: ignore
from markdownify import markdownify  # type: ignore
from md2gemini import md2gemini  # type: ignore
from readability import Document  # type: ignore
from requests.adapters import HTTPAdapter  # type: ignore
from urllib3.util.retry import Retry

from tenkan.utils import measure


class Feed:
    """
    receives various feed data and applies necessary changes to make it usable into files
    """

    def __init__(
        self,
        input_content: dict,
        filters=None,
        formatting=None,
    ) -> None:
        self.content = input_content
        self.filters = filters
        self.formatting = formatting
        self.new_entries: list = []

    def needs_update(self) -> bool:
        """Checks if updates are available"""
        if not self.content['json_hash_last_update']:
            return True
        if (
            self.content['json_hash_last_update']
            != self.content['fetched_hash_last_update']
        ):
            return True
        return False

    @measure
    def get_new_entries(self) -> None:
        """Selects new entries depending on filters defined on config file"""
        for entry in self.content['fetched_content']['entries']:
            if (
                any(
                    x in entry['title']
                    for x in self.filters.get('titles_blacklist', '').split(
                        ','
                    )
                )
                or any(
                    x in entry['link']
                    for x in self.filters.get('links_blacklist', '').split(',')
                )
                or any(
                    # feedparser object can be problematic sometimes
                    # we need to check if we have an authors item
                    # AND we check if we can get it's name because it can be empty
                    # AND if we don't have any of these, we return a stupid string
                    # to match the str type which is expected
                    x
                    in (
                        entry.get('authors')
                        and entry.authors[0].get('name')
                        or 'random string'
                    )
                    for x in self.filters.get('authors_blacklist', '').split(
                        ','
                    )
                )
            ):
                self.content['fetched_content']['entries'].remove(entry)
                continue
            self.new_entries.append(entry)

    @measure
    def export_content(self) -> dict:
        """Exports properly formatted content"""
        # create feed item structure
        data_export: dict[str, List] = {
            'title': self.content['title'],
            'last_update': self.content['last_update'],
            'gmi_url': self.content['gmi_url'],
            'articles': [],
            'hash_last_update': self.content['fetched_hash_last_update'],
        }
        for article in self.new_entries:
            article_formatted_title = self._format_article_title(article)
            article_date = self._get_article_date(article)

            # 2 possibilities to get content : content['value'] or summary
            content = (
                article['content'][0]['value']
                if article.get('content')
                else article['summary']
            )

            article_content = self._format_article_content(
                content, link=article['link']
            )

            data_export['articles'].append(
                {
                    'article_title': article['title'],
                    'article_formatted_title': article_formatted_title,
                    'article_content': article_content,
                    'article_date': article_date,
                    'http_url': article['link'],
                    'updated': article_date,
                }
            )

        return data_export

    @classmethod
    def _get_article_date(cls, article: dict) -> datetime:
        """get date string and return datetime object"""
        try:
            return (
                datetime(
                    *article.get(
                        'published_parsed', article['updated_parsed']
                    )[:6]
                )
                .replace(tzinfo=timezone.utc)
                .astimezone(tz=None)
            )
        except KeyError:
            logging.error(
                "Can't find a proper date field in article data, this should not happen !"
            )
            sys.exit(1)

    @measure
    def _format_article_title(self, article: dict) -> str:
        """title formatting to make it usable as a file title"""
        # truncate title size depending on title size
        maxlen = int(self.formatting.get('title_size', 120))
        if len(self.content['title']) + len(article['title']) > maxlen:
            maxlen = maxlen - len(self.content['title'])

        # We don't want multiline titles (yes, it happens)
        article['title'] = article['title'].replace('\n', '')[:maxlen]

        # remove special characters
        # probably not the best way to do it, as it seems there is performance issues here
        # to improve later if possible
        formatted_str = (
            article['title']
            .encode('utf8', 'ignore')
            .decode('utf8', 'ignore')
            .replace(' ', '-')
        )
        return re.sub('[«»!@#$%^&*(){};:,./<>?/|`~=_+]', '', formatted_str)[
            :maxlen
        ]

    @measure
    def _format_article_content(self, content: str, link: str) -> str:
        """
        Formats article content from html to gmi
        Will use readability if the feed is truncated, so it should retrieve the full content
        """

        # conversion to readability format if asked
        if self.content['title'] in self.formatting.get(
            'truncated_feeds', 'アケオメ'
        ).split(','):
            headers = {
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/42.0.2311.135 Safari/537.36 Edge/12.246'
            }

            req = requests.Session()
            retries = Retry(
                total=5,
                backoff_factor=0.1,
                status_forcelist=[500, 502, 503, 504],
            )
            req.mount('http://', HTTPAdapter(max_retries=retries))
            req.mount('https://', HTTPAdapter(max_retries=retries))
            res = req.get(url=link, headers=headers)

            content = Document(res.text).summary()

        # convert html -> md -> gemini
        article = md2gemini(markdownify(content))

        return article



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tenkan']

package_data = \
{'': ['*']}

install_requires = \
['DateTime>=4.3,<5.0',
 'feedgen>=0.9.0,<0.10.0',
 'feedparser>=6.0.8,<7.0.0',
 'markdownify>=0.10.0,<0.11.0',
 'md2gemini>=1.8.1,<2.0.0',
 'prettytable>=3.0.0,<4.0.0',
 'readability-lxml>=0.8.1,<0.9.0',
 'requests>=2.26.0,<3.0.0',
 'rich>=10.16.2,<11.0.0']

entry_points = \
{'console_scripts': ['tenkan = tenkan.cli:main']}

setup_kwargs = {
    'name': 'tenkan',
    'version': '0.1.1',
    'description': 'RSS/atom feed converter from html to gemini',
    'long_description': "# tenkan\n\nCommand line tool to convert HTTP RSS/Atom feeds to gemini format.\n\n## Installation\n```shell script\npip install tenkan\n```\n\n## Usage\n\nAdd a feed\n```shell script\n# Any valid RSS/Atom feed\ntenkan add feedname url\n```\n\nUpdate content of feed list\n```shell script\ntenkan update\n```\n\nDelete feed\n```shell script\ntenkan delete feedname\n```\n\nList subscripted feeds\n```shell script\ntenkan list\n```\n## Options\nA debug mode is avaible via --debug option.\nIf you want to use your configuration or feeds file in another place than default one, you can use --config and --feedsfile options.\n\n\n## Configuration\ntenkan searches for a configuration file at the following location:\n\n`$XDG_CONFIG_HOME/tenkan/tenkan.conf`\n\n### Example config\nThis can be found in tenkan.conf.example.\n\n```ini\n[tenkan]\ngemini_path = /usr/local/gemini/\ngemini_url = gemini://foo.bar/feeds/\n# will purge feed folders having more than defined element count\n# purge_feed_folder_after = 100\n\n[filters]\n# authors we don't want to read\n# authors_blacklist = foo, bar\n# blacklist of article titles, if provided, it won't be processed\n# titles_blacklist = foo, bar\n# blacklist of article links, if provided, it won't be processed\n# links_blacklist = foo/bar.com, bar/foo, bla\n\n[formatting]\n# maximum article title size, 120 chars if not provided\n# title_size = 120\n\n# feeds with a truncated content\n# will be fetched and converted using readability\n# truncated_feeds = foo, bar\n```\n\n## Todolist\n- [ ] Add a edit command\n- [ ] Add a --feedname option to update command, to update a single feed\n- [ ] Rewrite configuration checks\n- [ ] Improve tests\n- [ ] Refactor needed parts like write_article\n- [ ] (not sure if relevant) migrate images too, for gemini clients that can handle it\n\n## Development\nI recommend using pre-commit. The pre-commit configuration I use is located in .pre-commit-config.yamlfile.\n\nRun pre-commit command before every pull request and fix the warnings or errors it produces.\n",
    'author': 'Quentin Ferrand',
    'author_email': 'quentin.ferrand@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://git.fqserv.eu/takaoni/tenkan',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-

"""config module : configuration file parsing"""

import configparser
import logging
import sys
from pathlib import Path


def load_config(config_file) -> configparser.ConfigParser:
    """config load"""

    # exit with error if config file not found
    if not Path(config_file).exists():
        logging.error('No config file found %s, exiting', config_file)
        sys.exit(1)

    parser = configparser.ConfigParser()
    parser.read(config_file)
    if 'tenkan' not in parser.sections():
        logging.critical(
            "Missing [tenkan] section in config file %s, can't go further",
            config_file,
        )
        sys.exit(1)

    # shitty checks of config content
    # to improve later...
    for opt in ['gemini_path', 'gemini_url']:
        if not parser.has_option('tenkan', opt):
            logging.error('Missing option %s', opt)
            sys.exit(1)

    if parser.has_option('tenkan', 'purge_feed_folder_after'):
        if not int(parser['tenkan']['purge_feed_folder_after']):
            logging.error(
                'Wrong type for purge_feed_folder_after option, should be a number'
            )
            sys.exit(1)

    if parser.has_section('filters'):
        for item in parser['filters']:
            parser['filters'][item] = parser['filters'][item].replace(' ', '')

    if parser.has_option('formatting', 'truncated_feeds'):
        parser['formatting']['truncated_feeds'] = parser['formatting'][
            'truncated_feeds'
        ].replace(' ', '')

    if parser.has_option('formatting', 'title_size') and not int(
        parser['formatting']['title_size']
    ):
        logging.error('Wrong type for title_size option, should be a number')
        sys.exit(1)

    return parser
